using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace AlphaTestClock
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            alphaTestEffect = new AlphaTestEffect(GraphicsDevice);

            // Set up an AlphaTestEffect with a Projection that allows it to be used with SpriteBatch.
            Matrix projection = Matrix.CreateOrthographicOffCenter(0, GraphicsDevice.PresentationParameters.BackBufferWidth, GraphicsDevice.PresentationParameters.BackBufferHeight, 0, 0, 1);
            Matrix halfPixelOffset = Matrix.CreateTranslation(-0.5f, -0.5f, 0);

            alphaTestEffect = new AlphaTestEffect(GraphicsDevice);
            alphaTestEffect.VertexColorEnabled = false;
            alphaTestEffect.AlphaFunction = CompareFunction.Greater;
            alphaTestEffect.ReferenceAlpha = 0;
            alphaTestEffect.World = Matrix.Identity;
            alphaTestEffect.View = Matrix.Identity;
            alphaTestEffect.Projection = halfPixelOffset * projection;

            base.Initialize();
        }

        private AlphaTestEffect alphaTestEffect;
        private Texture2D clockTexture;

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            clockTexture = Content.Load<Texture2D>(@"Clock");
            Color[] colors = new Color[clockTexture.Width * clockTexture.Height];
            // Replace the alpha with a value that depends on the angle.
            clockTexture.GetData(colors);
            int xCenter = clockTexture.Width / 2;
            int yCenter = clockTexture.Height / 2;
            for (int x = 0; x < clockTexture.Width; x++)
            {
                for (int y = 0; y < clockTexture.Height; y++)
                {
                    int index = x + y * clockTexture.Width;
                    double angle = (Math.Atan2(y - yCenter, x - xCenter) + MathHelper.Pi) / MathHelper.TwoPi;
                    colors[index].A = (byte)(angle * 255);
                }
            }
            clockTexture.SetData(colors);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        private float time;
        private const float CycleSeconds = 2;

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            time += (float)gameTime.ElapsedGameTime.TotalSeconds;
            time %= CycleSeconds;

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);
            alphaTestEffect.ReferenceAlpha = (int)(time * 255 / CycleSeconds);
            spriteBatch.Begin(SpriteSortMode.Deferred, BlendState.Opaque, null, null, null, alphaTestEffect);
            spriteBatch.Draw(clockTexture, Vector2.Zero, Color.White);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
